"""
    This file is part of Accountant.

    Accountant is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Accountant is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with Accountant.  If not, see <http://www.gnu.org/licenses/>.
"""
from flask import json, request
from flask.ext.restful import Resource, fields, reqparse, marshal_with_field

from sqlalchemy.orm.exc import NoResultFound

from accountant import session_scope, session_aware

from . import auth
from .. import api, api_api

from ..models.accounts import Account
from ..models.entries import Entry
from ..models.operations import Operation

from ..fields import Object


@api.route("/accounts/<account_id>/<year>/<month>/")
@auth.login_required
def get_account_status(account_id, year, month):
    with session_scope() as session:
        query = Operation.get_account_status(session, account_id, year, month)

        if query.count() == 1:
            result = query.one()
            revenues = result.revenues
            expenses = result.expenses
            balance = result.balance
        else:
            revenues = 0.0
            expenses = 0.0
            balance = 0.0

        return json.dumps({
            "expenses": str(expenses),
            "revenues": str(revenues),
            "balance": str(balance)
        })


@api.route("/accounts/<account_id>/months")
@auth.login_required
def get_months(account_id):
    with session_scope() as session:
        query = Entry.get_months_for_account(session, account_id)

        return json.dumps([{
            "year": i.year,
            "month": i.month.rjust(2, '0')
        } for i in query.all()])


resource_fields = {
    'id': fields.Integer,
    'name': fields.String,
    'authorized_overdraft': fields.Fixed(decimals=2),
    'current': fields.Float,
    'pointed': fields.Float,
    'future': fields.Float,
}


parser = reqparse.RequestParser()
parser.add_argument('name', type=str, required=True)
parser.add_argument('authorized_overdraft', type=float, required=True)


class AccountListResource(Resource):
    @session_aware
    @marshal_with_field(fields.List(Object(resource_fields)))
    def get(self, session=None):
        """
        Returns accounts with their balances.
        """
        return Account.get_accounts(session).all(), 200

    def put(self):
        """
        Batch update, not implemented.
        """
        raise NotImplementedError()

    @session_aware
    @marshal_with_field(Object(resource_fields))
    def post(self, session=None):
        """
        Create a new account.
        """
        kwargs = parser.parse_args()

        account = Account(**kwargs)

        session.add(account)

        return account, 201

    def delete(self):
        """
        Batch delete, not implemented.
        """
        raise NotImplementedError()


class AccountResource(Resource):
    @session_aware
    @marshal_with_field(Object(resource_fields))
    def get(self, account_id, session=None):
        """
        Get account.
        """
        try:
            return Account.get(session, account_id)
        except NoResultFound:
            return None, 404

    @session_aware
    @marshal_with_field(Object(resource_fields))
    def delete(self, account_id, session=None):
        try:
            account = Account.get(session, account_id)
        except NoResultFound:
            return None, 404

        session.delete(account)

        return account

    def patch(self, id):
        pass

    @session_aware
    @marshal_with_field(Object(resource_fields))
    def put(self, account_id, session=None):
        kwargs = parser.parse_args()

        assert (id not in kwargs or kwargs.id is None
                or kwargs.id == account_id)

        try:
            account = Account.get(session, account_id)
        except NoResultFound:
            return None, 404

        # SQLAlchemy objects ignore __dict__.update() with merge.
        for k, v in kwargs.items():
            setattr(account, k, v)

        session.merge(account)

        return account


api_api.add_resource(AccountListResource, '/accounts')
api_api.add_resource(AccountResource, '/accounts/<int:account_id>')
