"""
    This file is part of Accountant.

    Accountant is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Accountant is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with Accountant.  If not, see <http://www.gnu.org/licenses/>.
"""
import dateutil.parser

from flask import json, request
from flask.ext.restful import Resource, fields, reqparse, marshal_with_field

from sqlalchemy.orm.exc import NoResultFound

from accountant import session_scope, session_aware

from ..models.scheduled_operations import ScheduledOperation

from .. import api, api_api

from ..fields import Object


@api.route("/scheduled_operations/<int:account_id>")
def get_scheduled_operations(account_id):
    """
    Return entries for an account, year, and month.
    """
    with session_scope() as session:
        query = ScheduledOperation.get_scheduled_operations_for_account(
            session, account_id)

        return json.dumps([{
            "id": i.id,
            "start_date": i.start_date.strftime("%Y-%m-%d"),
            "stop_date": i.stop_date.strftime("%Y-%m-%d"),
            "day": str(i.day),
            "frequency": str(i.frequency),
            "label": i.label,
            "value": str(i.value),
            "category": i.category,
            "account_id": i.account_id
        } for i in query.all()])


resource_fields = {
    'id': fields.Integer,
    'start_date': fields.DateTime(dt_format='iso8601'),
    'stop_date': fields.DateTime(dt_format='iso8601'),
    'day': fields.Integer,
    'frequency': fields.Integer,
    'label': fields.String,
    'value': fields.Fixed(decimals=2),
    'category': fields.String,
    'account_id': fields.Integer,
}


parser = reqparse.RequestParser()
parser.add_argument('start_date', type=lambda a: dateutil.parser.parse(a))
parser.add_argument('stop_date', type=lambda a: dateutil.parser.parse(a))
parser.add_argument('day', type=int)
parser.add_argument('frequency', type=int)
parser.add_argument('label', type=str)
parser.add_argument('value', type=float)
parser.add_argument('category', type=str)
parser.add_argument('account_id', type=int)


class ScheduledOperationListResource(Resource):
    @session_aware
    @marshal_with_field(Object(resource_fields))
    def put(self, session=None):
        """
        Add a new scheduled operation.
        """
        kwargs = parser.parse_args()

        scheduledOperation = ScheduledOperation(**kwargs)

        session.add(scheduledOperation)

        return scheduledOperation, 201


class ScheduledOperationResource(Resource):
    @session_aware
    @marshal_with_field(Object(resource_fields))
    def get(self, scheduled_operation_id, session=None):
        """
        Get scheduled operation.
        """
        try:
            return ScheduledOperation.get(session, scheduled_operation_id)
        except NoResultFound:
            return None, 404

    @session_aware
    @marshal_with_field(Object(resource_fields))
    def delete(self, scheduled_operation_id, session=None):
        try:
            scheduled_operation = ScheduledOperation.get(
                session, scheduled_operation_id)
        except NoResultFound:
            return None, 404

        session.delete(scheduled_operation)

        return scheduled_operation

    @session_aware
    @marshal_with_field(Object(resource_fields))
    def put(self, scheduled_operation_id, session=None):
        kwargs = parser.parse_args()

        assert (id not in kwargs or kwargs.id is None
                or kwargs.id == scheduled_operation_id)

        try:
            scheduled_operation = ScheduledOperation.get(
                session, scheduled_operation_id)
        except NoResultFound:
            return None, 404

        # SQLAlchemy objects ignore __dict__.update() with merge.
        for k, v in kwargs.items():
            setattr(scheduled_operation, k, v)

        session.merge(scheduled_operation)

        return scheduled_operation


api_api.add_resource(ScheduledOperationListResource, "/scheduled_operations")
api_api.add_resource(ScheduledOperationResource,
                     "/scheduled_operations/<int:scheduled_operation_id>")
